=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2007-11-29 22:24:49 +0000
+++ bzrlib/errors.py	2007-11-30 07:45:56 +0000
@@ -1332,7 +1332,8 @@
 
 
 class KnitDataStreamIncompatible(KnitError):
-    # Not raised anymore, as we can convert data streams
+    # Not raised anymore, as we can convert data streams.  In future we may
+    # need it again for more exotic cases, so we're keeping it around for now.
 
     _fmt = "Cannot insert knit data stream of format \"%(stream_format)s\" into knit of format \"%(target_format)s\"."
 

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2007-11-30 07:19:25 +0000
+++ bzrlib/knit.py	2007-11-30 07:45:56 +0000
@@ -1535,7 +1535,7 @@
             return 'line-delta'
 
     def get_options(self, version_id):
-        """Return a string representing options.
+        """Return a list representing options.
 
         e.g. ['foo', 'bar']
         """
@@ -2080,14 +2080,13 @@
     def get_raw_records(self, memos_for_retrieval):
         """Get the raw bytes for a records.
 
-        :param memos_for_retrieval: An iterable containing the (index, pos, 
-            length) memo for retrieving the bytes. The .knit method ignores
-            the index as there is always only a single file.
+        :param memos_for_retrieval: An iterable containing the (thunk_flag,
+            index, start, end) memo for retrieving the bytes.
         :return: An iterator over the bytes of the records.
         """
         # use a generator for memory friendliness
-        for index, start, end in memos_for_retrieval:
-            if index is self.stream_index:
+        for thunk_flag, version_id, start, end in memos_for_retrieval:
+            if version_id is self.stream_index:
                 yield self.data[start:end]
                 continue
             # we have been asked to thunk. This thunking only occurs when
@@ -2096,10 +2095,9 @@
             # We could improve performance here by scanning for where we need
             # to do this and using get_line_list, then interleaving the output
             # as desired. However, for now, this is sufficient.
-            if (index[:6] != 'thunk:' or
-                self.orig_factory.__class__ != KnitPlainFactory):
-                raise errors.KnitCorrupt(self, 'Bad thunk request %r' % index)
-            version_id = index[6:]
+            if self.orig_factory.__class__ != KnitPlainFactory:
+                raise errors.KnitCorrupt(
+                    self, 'Bad thunk request %r' % version_id)
             lines = self.backing_knit.get_lines(version_id)
             line_bytes = ''.join(lines)
             digest = sha_string(line_bytes)
@@ -2175,7 +2173,7 @@
             raise errors.KnitIndexUnknownMethod(self, options)
 
     def get_options(self, version_id):
-        """Return a string representing options.
+        """Return a list representing options.
 
         e.g. ['foo', 'bar']
         """
@@ -2189,17 +2187,18 @@
         """Return details needed to access the version.
         
         _StreamAccess has the data as a big array, so we return slice
-        coordinates into that (as index_memo's are opaque outside the 
-        index and matching access class.
+        coordinates into that (as index_memo's are opaque outside the
+        index and matching access class).
 
-        :return: a tuple (None, start, end).
+        :return: a tuple (thunk_flag, index, start, end).  If thunk_flag is
+            False, index will be self, otherwise it will be a version id.
         """
         try:
             start, end = self._by_version[version_id][1]
-            return self, start, end
+            return False, self, start, end
         except KeyError:
             # Signal to the access object to handle this from the backing knit.
-            return ('thunk:%s' % version_id, None, None)
+            return (True, version_id, None, None)
 
     def get_versions(self):
         """Get all the versions in the stream."""

=== modified file 'bzrlib/tests/test_knit.py'
--- bzrlib/tests/test_knit.py	2007-11-30 07:19:25 +0000
+++ bzrlib/tests/test_knit.py	2007-11-30 07:45:56 +0000
@@ -1784,7 +1784,9 @@
             knit2.transport.get_bytes(knit2._index._filename))
 
     def assertKnitValuesEqual(self, left, right):
-        """Assert that the texts and graph of left and right are the same."""
+        """Assert that the texts, annotations and graph of left and right are
+        the same.
+        """
         self.assertEqual(set(left.versions()), set(right.versions()))
         for version in left.versions():
             self.assertEqual(left.get_parents_with_ghosts(version),
@@ -2865,8 +2867,8 @@
 
     def assertGetPosition(self, knit, versions, version, result):
         index = self.get_index(knit, knit.get_data_stream(versions))
-        if result[0] is None:
-            result = (index, result[1], result[2])
+        if result[1] is None:
+            result = (result[0], index, result[2], result[3])
         self.assertEqual(result, index.get_position(version))
 
     def assertGetParentsWithGhosts(self, knit, versions, version, parents):
@@ -2951,14 +2953,14 @@
 
     def test_get_position(self):
         knit = self.make_knit_with_4_versions_2_dags()
-        # get_position returns (index(can be None), start, end) for
+        # get_position returns (thunk_flay, index(can be None), start, end) for
         # _StreamAccess to use.
-        self.assertGetPosition(knit, ['a'], 'a', (None, 0, 78))
-        self.assertGetPosition(knit, ['a', 'c'], 'c', (None, 78, 156))
+        self.assertGetPosition(knit, ['a'], 'a', (False, None, 0, 78))
+        self.assertGetPosition(knit, ['a', 'c'], 'c', (False, None, 78, 156))
         # get_position on a text that is not in the datastream (but in the
-        # backing knit) returns ('thunk:versionid', None, None) - and then the
+        # backing knit) returns (True, 'versionid', None, None) - and then the
         # access object can construct the relevant data as needed.
-        self.assertGetPosition(knit, ['a', 'c'], 'b', ('thunk:b', None, None))
+        self.assertGetPosition(knit, ['a', 'c'], 'b', (True, 'b', None, None))
 
 
 class Test_StreamAccess(KnitTests):
@@ -3015,7 +3017,7 @@
         target_knit.join(source_knit)
         index, access = self.get_index_access(target_knit,
             source_knit.get_data_stream([]))
-        raw_data = list(access.get_raw_records([("thunk:A", None, None)]))[0]
+        raw_data = list(access.get_raw_records([(True, "A", None, None)]))[0]
         df = GzipFile(mode='rb', fileobj=StringIO(raw_data))
         self.assertEqual(
             'version A 1 5d36b88bb697a2d778f024048bafabd443d74503\n'
@@ -3028,4 +3030,4 @@
         index, access = self.get_index_access(knit,
             knit.get_data_stream([]))
         self.assertRaises(errors.KnitCorrupt,
-            list, access.get_raw_records([("thunk:A", None, None)]))
+            list, access.get_raw_records([(True, "A", None, None)]))

